!> \file sliding-Heino_mod.f90
!! Module qui definit la loi de glissement dans les experiences Heino.
!<

!> \namespace  sliding_dragging_heino
!! Definition de la loi de glissement dans les experiences Heino.
!! \author ...
!! \date ...
!! @note Used module
!! @note   - use module3D_phy
!<

module sliding_dragging_heino

! Dfini la loi de glissement dans les expriences Heino

USE module3D_phy


implicit none


integer,dimension(nx,ny) :: mksedim      !< masque des regions sediment-hard rock
integer,dimension(nx,ny) :: mkxsedim     !< dcalle sur les demi mailles
integer,dimension(nx,ny) :: mkysedim     !< dcalle sur les demi mailles
real :: Cr                            !< coefficient glissement pour hard rock
real :: Cs                            !< coefficient glissement pour sediment
real :: coefmax
real :: coefslid
real :: coefdrag
!logical,dimension(nx,ny) :: gzmx_heino     !< pour transporter la valeur dans flottab
!logical,dimension(nx,ny) :: gzmy_heino
real :: longslope
real :: seuil_gliss                   !< seuil sur hw_mx pour avoir du glissement
real, dimension(nx,ny) :: hw_mx
real, dimension(nx,ny) :: hw_my

character(len=80) :: filin


contains
!-------------------------------------------------------------------------------
!> SUBROUTINE: init_sliding
!! Cette routine fait l'initialisation du glissement et est appele juste apres
!! la lecture du masque
!<
  subroutine init_sliding

    if (itracebug.eq.1)  call tracebug(' Heino: entree dans routine init_sliding')

    ! Cette routine fait l'initialisation du glissement et est appele juste apres
    ! la lecture du masque

    ! Cette routine fait l'initialisation du glissement

    loigliss=4       ! 4 -> que glissement Heino (pas de mode dragging)
    if (loigliss.eq.4) then
       flgzmx(:,:)=.false.
       flgzmy(:,:)=.false.
    endif
    coefmax=hwatermax
    seuil_gliss=0.0001
    ! 5 -> glissement Heino + dragging

    !    ecriture dans le fichier parametres
    write(num_rep_42,*)'glissement Heino -  initialisation masque'
    write(num_rep_42,*)'loi glissement=',loigliss
    !     write(num_rep_42,*)'coef=1, gliss si les 2 voisins temperes' 
    write(num_rep_42,*)'glissement si eau basale (6 voisins) >=seuil_gliss=',seuil_gliss
    write(num_rep_42,*)'coef dependant de la hauteur d eau    coefmax=',coefmax
    write(num_rep_42,*)' moyenne de la hauteur d eau a partir de 6 voisins'
    write(num_rep_42,*)'----------------------------------------------------------'


    ! fichier mask fourni par l'intercomparaison Heino 
    ! on refait la lecture 

    filin='mask.dat'
    filin = TRIM(DIRNAMEINP)//TRIM(filin)

    write(num_rep_42,*) 'type de socle '
    write(num_rep_42,*) 'mksedim donne par ',filin

    open(num_coupe,file=filin)
    do i=1,6
       read(num_coupe,*)  ! 6 lignes de commentaires
    end do

    ! lecture proprement dite de la carte sediment
    do j=ny, 1, -1
       read(num_coupe,'(81i1)') (mksedim(i,j), i=1,nx)
    end do

    close(num_coupe)


    ! test pour une calotte axi-symetrique avec glissement Cr partout
    !write(num_rep_42,*) 'masque sediment mis a 1 '
    !mksedim(:,:)=min(mksedim(:,:),1)

    ! a la frontiere le demi noeud est sediment.
    ! calcul du masque sur les demi mailles : frontiere -> sedim
    mkxsedim(:,:)=max(mksedim(:,:),eoshift(mksedim(:,:),shift=-1,boundary=0,dim=1))
    mkysedim(:,:)=max(mksedim(:,:),eoshift(mksedim(:,:),shift=-1,boundary=0,dim=2))
    write(num_rep_42,*) 'sur les bords zone sediment, masque exterieur '


    !mkxsedim(:,:)=min(mksedim(:,:),eoshift(mksedim(:,:),shift=-1,boundary=0,dim=1))
    !mkysedim(:,:)=min(mksedim(:,:),eoshift(mksedim(:,:),shift=-1,boundary=0,dim=2))

    !write(num_rep_42,*) 'sur les bords zone sediment, demi-maille sediment si les 2 mailles le sont'

    mslid_mx(:,:)=mkxsedim(:,:)
    mslid_my(:,:)=mkysedim(:,:)

    Cr=1.e5    ! en a-1  coefficient hard rock (loi en puissance 3)
    Cs=500.    ! en a-1  coefficient sediment (lineaire)


    write(num_rep_42,*)'coefficients:   Cr=',Cr,'    Cs=',Cs,' (a-1)'
    !  write(num_rep_42,*)'glissement seulement si les 2 voisins sont au point de fusion'



    return
  end subroutine init_sliding

!________________________________________________________________________________

!> SUBROUTINE: init_dragging
!! Initialisation du basal dragging
!<
subroutine init_dragging



if ((loigliss.eq.5).and.(Cs.gt.1.)) then
   coefdrag=rog/Cs
   Cs=0.
   write(num_rep_42,*)'dragging Heino'
   write(num_rep_42,*)'passage en stream si zone sediment + fusion'
   write(num_rep_42,*)'coefficient de frottement dans la zone sediment (Pa/m)=',coefdrag
endif

end subroutine init_dragging

!________________________________________________________________________________

!> SUBROUTINE: sliding
!! Cette routine calcule le terme ddbx et ddby dans le cas Heino
!<
subroutine sliding


! cette routine calcule le terme ddbx et ddby dans le cas Heino
!      -------------------------------------------------
!      GLISSEMENT
!      DDBX et DDBY termes dus au glissement 
!      relation avec la vitesse de glissement UXB et UYB 
!      UXB=-DDBX*SDX et UYB=-DDBY*SDY
!      -------------------------------------------------

implicit none

integer :: i_moins1,j_moins1,i_plus1,j_plus1

!!$real, dimension(nx,ny) :: hw_mx
!!$real, dimension(nx,ny) :: hw_my

if (itracebug.eq.1)  call tracebug(' Heino: entree dans routine sliding')


call moy_mxmy(nx,ny,Hwater,hw_mx,hw_my)



slidy: do j=2,NY
   do I=2,NX-1

pose_y:  if (.not.flgzmy(i,j)) then

   if (HMY(I,J).lt.1.) then        ! glace tres fine
      ddby(i,j)=0.
      mslid_my(i,j)=0

!   else if ((T(i,j,nz).lt.Tpmp(i,j,nz)).and.(T(i,j-1,nz).lt.Tpmp(i,j-1,nz))) then 
!      ddby(i,j)=0.                 ! pas de glissement seulement
!                                   ! si les 2 voisins sont froids

!   else if ((T(i,j,nz).lt.Tpmp(i,j,nz)).or.(T(i,j-1,nz).lt.Tpmp(i,j-1,nz))) then 

!      else if ((ibase(i,j).eq.1).or.(ibase(i,j-1).eq.1)) then
!      else if ((ibase(i,j).eq.1).and.(ibase(i,j-1).eq.1)) then

      else if (hw_my(i,j).lt.seuil_gliss) then
      ddby(i,j)=0.                 ! pas de glissement si tous les voisins 
                                   ! sont a sec
      mslid_my(i,j)=0
      flgzmy(i,j)=.false.

   else                            ! glissement
!      coefslid=(hwater(i,j)+hwater(i,j-1))*0.5
      coefslid=hw_my(i,j)
      coefslid=max(coefslid,0.)
      coefslid=min(coefslid/coefmax,1.)
!      coefslid=1.

      if (mkysedim(i,j).eq.1) then  ! loi hard rock
         ddby(i,j)=Cr*slope2my(i,j)*Hmy(i,j)*coefslid

         mslid_my(i,j)=1

      else if ((mkysedim(i,j).eq.2).and.(loigliss.eq.4)) then  ! loi sediment
         ddby(i,j)=Cs*Hmy(i,j)*coefslid

         mslid_my(i,j)=2

!         if (abs(sdy(i,j)).gt.1.e-8) then
!            j_1=max(j-1,1)
!            j1=min(j+1,ny)
!            longslope=(sdy(i,j_1)+sdy(i,j)+sdy(i,j1))/3.
!            ddby(i,j)=ddby(i,j)/sdy(i,j)*longslope
!         endif

      else
         ddby(i,j)=0.
         mslid_my(i,j)=0

      endif

   endif


endif pose_y
end do
end do slidy


slidx: do j=2,NY-1
   do I=2,NX
pose_x:  if (.not.flgzmx(i,j)) then

   if (HMx(I,J).lt.1.) then        ! glace tres fine
      ddbx(i,j)=0.
         mslid_mx(i,j)=0

!   else if ((T(i,j,nz).lt.Tpmp(i,j,nz)).and.(T(i-1,j,nz).lt.Tpmp(i-1,j,nz))) then 
!      ddbx(i,j)=0.                 ! pas de glissement seulement
!                                   ! si les 2 voisins sont froids

!   else if ((T(i,j,nz).lt.Tpmp(i,j,nz)).or.(T(i-1,j,nz).lt.Tpmp(i-1,j,nz))) then 
!     else if ((ibase(i,j).eq.1).or.(ibase(i-1,j).eq.1)) then

!      else if ((ibase(i,j).eq.1).and.(ibase(i-1,j).eq.1)) then
      else if (hw_mx(i,j).lt.seuil_gliss) then

      ddbx(i,j)=0.                 ! pas de glissement si au moins un seul 
                                   ! des 2 voisins est froid
      mslid_mx(i,j)=0
      flgzmx(i,j)=.false.


   else                            ! glissement

!      coefslid=(hwater(i,j)+hwater(i-1,j))*0.5
      coefslid=hw_mx(i,j)
      coefslid=max(coefslid,0.)
      coefslid=min(coefslid/coefmax,1.)
!      coefslid=1.

      if (mkxsedim(i,j).eq.1) then  ! loi hard rock
         ddbx(i,j)=Cr*slope2mx(i,j)*Hmx(i,j)*coefslid
         mslid_mx(i,j)=1

      else if ((mkxsedim(i,j).eq.2).and.(loigliss.eq.4)) then  ! loi sediment
         ddbx(i,j)=Cs*Hmx(i,j)*coefslid
         mslid_mx(i,j)=2

!         if (abs(sdx(i,j)).gt.1.e-8) then
!            i_1=max(i-1,1)
!            i1=min(i+1,nx)
!            longslope=(sdx(i_1,j)+sdx(i,j)+sdx(i1,j))/3.
!            ddbx(i,j)=ddbx(i,j)/sdx(i,j)*longslope
!         endif


      else
         ddbx(i,j)=0.
         mslid_mx(i,j)=0

      endif

   endif

endif pose_x
end do
end do slidx

return
end subroutine  sliding

!-------------------------------------------------------------------------

!> SUBROUTINE: dragging
!! Definit le basal drag
!<

subroutine dragging   

! dans la zone sediment : appele si loigliss=5,
! defini le basal drag


flgzmx(:,:)=.false.
flgzmy(:,:)=.false.

betamx(:,:)=1.e5
betamy(:,:)=1.e5

shelfy=.false.

if (loigliss.eq.5) then          ! dragging dans les regions mkysedim:2

call moy_mxmy(nx,ny,Hwater,hw_mx,hw_my)

   froty: do j=2,ny
      do i=2,nx-1

         if (mkysedim(i,j).eq.2) then  ! loi sediment

            ! seulement si au dessus du point de fusion 
!            if ((T(i,j,nz).lt.Tpmp(i,j,nz)).or.(T(i,j-1,nz).lt.Tpmp(i,j-1,nz))) then 
            if (hw_my(i,j).lt.seuil_gliss) then
            ! base froide
            else
               coefslid=hw_my(i,j)
               coefslid=max(coefslid,seuil_gliss)
               coefslid=min(coefslid/coefmax,1.)

               betamy(i,j)=min(coefdrag/coefslid,1.e5)

!               betamy(i,j)=coefdrag


               flgzmy(i,j)=.true.
               ddby(i,j)=0.
               mslid_my(i,j)=5
               shelfy=.true.
            endif
         endif

      end do
   end do froty


   frotx: do j=2,ny-1
      do i=2,nx

         if (mkxsedim(i,j).eq.2) then  ! loi sediment

            ! seulement si au dessus du point de fusion 
!            if ((T(i,j,nz).lt.Tpmp(i,j,nz)).or.(T(i-1,j,nz).lt.Tpmp(i-1,j,nz))) then 
               if (hw_mx(i,j).lt.seuil_gliss) then
            ! base froide
            else
               coefslid=hw_mx(i,j)
               coefslid=max(coefslid,seuil_gliss)
               coefslid=min(coefslid/coefmax,1.)
               betamx(i,j)=min(coefdrag/coefslid,1.e5)


!               betamx(i,j)=coefdrag

               flgzmx(i,j)=.true.
               ddbx(i,j)=0.
               mslid_mx(i,j)=5
               shelfy=.true.
            endif

         endif
      end do
   end do frotx

   gzmx(:,:)=flgzmx(:,:)
   gzmy(:,:)=flgzmy(:,:)
!   gzmx_heino(:,:)=gzmx(:,:)
!   gzmy_heino(:,:)=gzmy(:,:)



end if

return
end subroutine dragging


END MODULE  sliding_dragging_heino


