!> \file sealevel-out.f90
!! Module pour les sorties des variations du niveau de la mer
!<

!> SUBROUTINE: sealevel_out
!! \author Vince 
!! \date 17 janvier 2001
!! @note Sorties des variations du niveau de la mer : nouveau calcul par rapport au niveau actuel
!! @note Pour executer cette routine on fait appel aux conditions actuels contenues dans un fichier pl
!! @note Used modules:
!! @note    - use module3D_phy
!! @note    - use param_phy_mod
!!
!<
subroutine sealevel_out

  USE module3D_phy
  USE param_phy_mod
  implicit none

  !     declarations
  real :: seal2
  real :: VOLF1,VOLF2,VOLG1,VOLG2,DX_DY
  real :: volsea ! difference entre la calotte au temps t et la calotte du fichier pl (etat actuel comme reference). la difference est exprimee en volume de glace.

  real :: V00,V10,V01,V3,V4,V01bis,V01ter
  integer :: NGR1,NGR2,N00,N11,N01,N10

  !     parametres
  DX_DY=DX*DY
  seal2=0    ! niveau de la mer actuel

  !   l' ouverture et la lecture du fichier pl de l'etat actuel (reference)
  !   se fait dans inputfile lecture de M_sealev, B_sealev, H_sealev et S_sealev
  !   'present1+k000.pl'  ! nom du fichier etat actuel 



  seal2=0    ! niveau de la mer actuel



  !      Mise a 0 des compteurs
  NGR1=0
  NGR2=0    
  VOLF1=0.
  VOLF2=0.
  VOLG1=0.
  VOLG2=0.
  VOLSEA=0.
  V00=0.
  V10=0.
  V01=0.
  V3=0.
  V4=0.
  V01bis=0.
  V01ter=0.
  N11=0
  N00=0
  N01=0
  N10=0

  !     debut de la boucle sur les points de grille  
  !      do k=1,np      
  do j=1,ny
     do i=1,nx
        !     Diagnostics sur les valeurs au moment t
        if (MK(i,j).eq.0) then     ! grounded
           NGR1=NGR1+1
           VOLG1=VOLG1+H(i,j) 

           !        calcul de la hauteur au dessus de la flottaison
           if (SEALEVEL-B(i,j).le.0.) then  ! socle au dessus niveau des mers du moment
              VOLF1=VOLF1+H(i,j) 
              ! on ajoute l'epaisseur de glace
           else ! socle en dessous du nivx des mers du moment
              VOLF1=VOLF1+H(i,j)-ROW/RO*(SEALEVEL-B(i,j))
              ! on ajoute l'epaisseur au-dessus du nivx de la mer plus
              ! le surplus de volume occupe par la glace dans l'eau
           endif
        endif
        !     Diagnostics sur le fichier actuel
        !      read(112,*) S2,H2,B2,M2
        if (M_sealev(i,j).eq.0) then     ! grounded
           NGR2 = NGR2 + 1
           VOLG2 = VOLG2 + H_sealev(i,j) 

           !        calcul de la hauteur au dessus de la flottaison
           if (SEAL2-B_SEALEV(I,J).le.0.) then  ! socle au dessus niveau des mers du moment
              VOLF2=VOLF2+H_SEALEV(I,J)
           else
              VOLF2=VOLF2+H_SEALEV(I,J)-ROW/RO*(SEAL2-B_SEALEV(I,J))
           endif
        endif

        !      Tests croises
        V4=V4+H(i,j)-H_SEALEV(I,J)
        if ((MK(i,j).eq.1).and.(M_SEALEV(I,J).eq.1)) then    ! flottant aux deux epoques
           VOLSEA=VOLSEA
           N11=N11+1
           !          write(13,*) i,j,0.

        else if ((MK(i,j).eq.0).and.(M_SEALEV(I,J).eq.0)) then  ! grounded aux deux epoques
           VOLSEA=VOLSEA+(H(i,j)-H_SEALEV(I,J))
           V00=V00+H(i,j)-H_SEALEV(I,J)
           V3=V3+H(i,j)-H_SEALEV(I,J)
           N00=N00+1
           !          write(13,*) i,j,H1-H_SEALEV(I,J)
        else if ((MK(i,j).eq.0).and.(M_SEALEV(I,J).eq.1)) then  ! grounded passe,float present 
           VOLSEA=VOLSEA+H(i,j)-ROW/RO*(SEAL2-B_SEALEV(I,J))    
           V01=V01+H(i,j)-ROW/RO*(SEAL2-B_SEALEV(I,J))
           !         write(6,*) H1,H_SEALEV(I,J),B1,B_SEALEV(I,J),H1-ROW/RO*(SEAL2-B_SEALEV(I,J))
           V01bis=V01bis+H(i,j)
           V01ter=V01ter+(SEAL2-B_SEALEV(I,J))*ROW/RO
           V3=V3+H(i,j)  
           N01=N01+1 
           !          write(13,*) i,j,H1-ROW/RO*(SEAL2-B_SEALEV(I,J))   
        else if ((MK(i,j).eq.1).and.(M_SEALEV(I,J).eq.0)) then  ! float passe, grounded present
           VOLSEA=VOLSEA-(H_SEALEV(I,J)-ROW/RO*(SEAL2-B_SEALEV(I,J)))
           V10=V10-(H_SEALEV(I,J)-ROW/RO*(SEAL2-B_SEALEV(I,J)))
           V3=V3-H_SEALEV(I,J)
           N10=N10+1
           !           write(13,*) i,j,H_SEALEV(I,J)-ROW/RO*(SEAL2-B_SEALEV(I,J))     
        endif
        !      if (MK(i,j).eq.0) then
        !             write(14,*) i,j,H1-H_SEALEV(I,J) 
        !      else
        !             write(14,*) i,j,0.
        !      endif

     end do
  end do         ! fin de la boucle sur les points de grille




  !     sortie des resultats   
  !write(num_sealevel,*) 'pas de temps'  
  !write(num_sealevel,*) 'file1', sealevel
  !write(num_sealevel,*) NGR1, VOLG1*dx_dy,VOLF1*dx_dy
  !write(num_sealevel,*)


  !write(num_sealevel,*) 'file2', seal2
  !write(num_sealevel,*) NGR2, VOLG2*dx_dy,VOLF2*dx_dy
  !write(num_sealevel,*)

  !write(num_sealevel,*) 'differences'
  !write(num_sealevel,*) (NGR1-NGR2)*dx_dy, (VOLG1-VOLG2)*dx_dy,(VOLF1-VOLF2)*dx_dy
  !write(num_sealevel,*)  VOLSEA*dx_dy,V00*dx_dy,V01*dx_dy,V10*dx_dy,V3*dx_dy
  !write(num_sealevel,*)  V4*dx_dy,V01bis*dx_dy,V01ter*dx_dy
  !write(num_sealevel,*)  N11,N00,N01,N10
  !write(num_sealevel,*)
  !write(num_sealevel,*)




  write(num_sealevel,112) time,(NGR1-NGR2)*dx_dy,(VOLG1-VOLG2)*dx_dy, &
       (VOLF1-VOLF2)*dx_dy,VOLSEA*dx_dy,V01ter*dx_dy
112 format(1x,f8.0,5(1x,e14.6))


end subroutine sealevel_out
