!> \file advec_icetemp_3D.f90
!! Implementation of Routines to calculate advection in the heat transfer equation
!! for a column
!<


!> SUBROUTINE: Advec_icetemp_3d
!! Calcule l'advection
!! Used modules:
!!    - use Icetemp_declar
!!    - use geom_type
!!    - use temperature_type
!!    - use Ice_flow_type
!!    - use Autre_pr_temp_type
!!    - use step_temp_type
!>

Subroutine Advec_icetemp_3d (Ii,Jj,Prodq_m,Cro_m,Advecx_m,Advecy_m,Advec_m,Ct_m,   &
     geom_m,T_m,Ice_flow_m,Autre_pr_temp_m,step_temp_m)


  use icetemp_declar 
  use geom_type
  use temperature_type
  use Ice_flow_type
  use Autre_pr_temp_type
  use step_temp_type

  Implicit None
  !< Arguments
  Integer,                    Intent(In)  ::  Ii,Jj          !< Indice Selon X Et Y
  Real, Dimension(:),         Intent(In)   :: Prodq_m        !< Tableau 1d Vert. heat production
  Real, Dimension(:),         Intent(In)   :: Cro_m          !< Tableau 1d Vert. heat capcity
  Real, Dimension(:),         Intent(Inout):: Advecx_m       !< Advection selon x
  Real, Dimension(:),         Intent(Inout):: Advecy_m       !< Advection selon y
  Real, Dimension(:),         Intent(Inout):: Advec_m        !< Advection total
  Real, Dimension(:),         Intent(In)   :: Ct_m           !< Tableau 1d Vert.  thermal cond.

  type (Geom),               intent(in)    :: geom_m         !variable geometrie global
  type (Ice_flow),           intent(in)    :: Ice_flow_m     !variable ice_flow global 
  type (Temperature),        intent(inout) :: T_m            !variable temperature global
  type (Autre_pr_temp),      intent(inout) :: Autre_pr_temp_m  
  type (step_temp),          intent(in)    :: step_temp_m    ! time steps

  

  Do K=2,Nz_m-1


     ! On Remplit Deux Tableaux Pour eviter Des Problemes D'Asymetrie
     !         Avection Selon X  (advecx_m en general > 0)
     !         ----------------
     Advecx_m(K) = Iadvec_w(Ii,Jj) * Ice_flow_m%Veloc_x(Ii,Jj,K) *          & ! ux west if upwind
                   (T_m%Temperature(Ii,Jj,K) - T_m%Temperature(Ii-1,Jj,K))  & ! west T gradient

                   + Iadvec_e(Ii+1,Jj)* Ice_flow_m%Veloc_x(Ii+1,Jj,K)*      & ! ux east if upwind
                    (T_m%Temperature(Ii+1,Jj,K)-T_m%Temperature(Ii,Jj,K))     ! east T gradient

     Advecx_m(K) =  Advecx_m(K) * Dx11                                        !Dx11=1/Dx


     !         Avection Selon Y
     !         ----------------
     Advecy_m(K) =   Iadvec_s(Ii,Jj) * Ice_flow_m%Veloc_y(Ii,Jj,K) *        & ! uy sud if upwind
                     (T_m%Temperature(Ii,Jj,K) - T_m%Temperature(Ii,Jj-1,K))& ! south T gradient

                    + Iadvec_n(Ii,Jj+1) * Ice_flow_m%Veloc_y(Ii,Jj+1,K) *   & ! uy nord is upwind
                     (T_m%Temperature(Ii,Jj+1,K)-T_m%Temperature(Ii,Jj,K))    ! north T gradient

     Advecy_m(K) =  Advecy_m(K) * Dx11

     Advec_m(K)=Advecx_m(K)+Advecy_m(K)  ! Advection Totale

  End Do

  !     Fin Boucle K, Ecriture Matrice Tridiag Dans La Glace


  !      -----------------------------Cas General (H>10m)
thick_ice:  If (Geom_m%Thickness(Ii,Jj).Gt.10.) Then

     !        Variables De Calcul Dans La Glace
     ! dou = dtt/dz^2
     Dou=step_temp_m%Dtt/De/De/ Geom_m%Thickness(Ii,Jj) / Geom_m%Thickness(Ii,Jj)  

     ! Dah = dtt/dz
     Dah=step_temp_m%Dtt /De /Geom_m%Thickness(Ii,Jj)

     ! thermal conductivity at mid point just below the surface                   
     Ct_bas=2.*(Ct_m(1)*Ct_m(2))/(Ct_m(1)+Ct_m(2))           

     ! surface temperature : cannot go above 0 celsius
     T_m%Temperature(Ii,Jj,1)=Min(0.,Autre_pr_temp_m%Ts(Ii,Jj))

! surface boundary condition
     Aa(1)=0.
     Bb(1)=1.
     Cc(1)=0.
     Rr(1)=T_m%Temperature(Ii,Jj,1)

     Do K=2,Nz_m-1
        Dzz=Dou/Cro_m(K)

        ! Conductivite Au Milieu Des Mailles
        Ct_haut=Ct_bas
        Ct_bas=2.*(Ct_m(K)*Ct_m(K+1))/(Ct_m(K)+Ct_m(K+1))

        ! Advection Verticale Centree
        Aa(K) = -Dzz * Ct_haut - Ice_flow_m%Veloc_z_r(Ii,Jj,K) * Dah / 2.  ! lower diag
        Cc(K) = -Dzz * Ct_bas  + Ice_flow_m%Veloc_z_r(Ii,Jj,K) * Dah / 2.  ! upper diag
        Bb(K) =  1.+ Dzz * (Ct_haut+Ct_bas)                                ! diag

        Rr(K) =  T_m%Temperature(Ii,Jj,K) + Prodq_m(K) * step_temp_m%Dtt   ! vector
        Rr(K) =  Rr(K)- Dttdx * (Advec_m(K))

     End DO
  End If thick_ice


End Subroutine Advec_icetemp_3d

!-------------------------------------------------------------------------


